==> rippled-1.1.2.tar.gz: rippled-1.1.2/src/ripple/app/main/NodeStoreScheduler.cpp <==
//------------------------------------------------------------------------------
/*
    This file is part of rippled: https://github.com/ripple/rippled
    Copyright (c) 2012, 2013 Ripple Labs Inc.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose  with  or without fee is hereby granted, provided that the above
    copyright notice and this permission notice appear in all copies.

    THE  SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
    WITH  REGARD  TO  THIS  SOFTWARE  INCLUDING  ALL  IMPLIED  WARRANTIES  OF
    MERCHANTABILITY  AND  FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
    ANY  SPECIAL ,  DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
    WHATSOEVER  RESULTING  FROM  LOSS  OF USE, DATA OR PROFITS, WHETHER IN AN
    ACTION  OF  CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
//==============================================================================

#include <ripple/app/main/NodeStoreScheduler.h>
#include <cassert>

namespace ripple {

NodeStoreScheduler::NodeStoreScheduler (Stoppable& parent)
    : Stoppable ("NodeStoreScheduler", parent)
{
}

void NodeStoreScheduler::setJobQueue (JobQueue& jobQueue)
{
    m_jobQueue = &jobQueue;
}

void NodeStoreScheduler::onStop ()
{
}

void NodeStoreScheduler::onChildrenStopped ()
{
    assert (m_taskCount == 0);
    stopped ();
}

void NodeStoreScheduler::scheduleTask (NodeStore::Task& task)
{
    ++m_taskCount;
    if (!m_jobQueue->addJob (
        jtWRITE,
        "NodeObject::store",
        [this, &task] (Job&) { doTask(task); }))
    {
        // Job not added, presumably because we're shutting down.
        // Recover by executing the task synchronously.
        doTask (task);
    }
}

void NodeStoreScheduler::doTask (NodeStore::Task& task)
{
    task.performScheduledTask ();

    // NOTE: It feels a bit off that there are two different methods that
    // call stopped(): onChildrenStopped() and doTask().  There's a
    // suspicion that, as long as the Stoppable tree is configured
    // correctly, this call to stopped() in doTask() can never occur.
    //
    // However, until we increase our confidence that the suspicion is
    // correct, we will leave this code in place.
    if ((--m_taskCount == 0) && isStopping() && areChildrenStopped())
        stopped();
}

void NodeStoreScheduler::onFetch (NodeStore::FetchReport const& report)
{
    if (report.wentToDisk)
        m_jobQueue->addLoadEvents (
            report.isAsync ? jtNS_ASYNC_READ : jtNS_SYNC_READ,
                1, report.elapsed);
}

void NodeStoreScheduler::onBatchWrite (NodeStore::BatchWriteReport const& report)
{
    m_jobQueue->addLoadEvents (jtNS_WRITE,
        report.writeCount, report.elapsed);
}

} // ripple

==> rippled-1.6.0.tar.gz: rippled-1.6.0/src/ripple/app/main/NodeStoreScheduler.cpp <==
//------------------------------------------------------------------------------
/*
    This file is part of rippled: https://github.com/ripple/rippled
    Copyright (c) 2012, 2013 Ripple Labs Inc.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose  with  or without fee is hereby granted, provided that the above
    copyright notice and this permission notice appear in all copies.

    THE  SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
    WITH  REGARD  TO  THIS  SOFTWARE  INCLUDING  ALL  IMPLIED  WARRANTIES  OF
    MERCHANTABILITY  AND  FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
    ANY  SPECIAL ,  DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
    WHATSOEVER  RESULTING  FROM  LOSS  OF USE, DATA OR PROFITS, WHETHER IN AN
    ACTION  OF  CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
//==============================================================================

#include <ripple/app/main/NodeStoreScheduler.h>
#include <cassert>

namespace ripple {

NodeStoreScheduler::NodeStoreScheduler(Stoppable& parent)
    : Stoppable("NodeStoreScheduler", parent)
{
}

void
NodeStoreScheduler::setJobQueue(JobQueue& jobQueue)
{
    m_jobQueue = &jobQueue;
}

void
NodeStoreScheduler::onStop()
{
}

void
NodeStoreScheduler::onChildrenStopped()
{
    assert(m_taskCount == 0);
    stopped();
}

void
NodeStoreScheduler::scheduleTask(NodeStore::Task& task)
{
    ++m_taskCount;
    if (!m_jobQueue->addJob(jtWRITE, "NodeObject::store", [this, &task](Job&) {
            doTask(task);
        }))
    {
        // Job not added, presumably because we're shutting down.
        // Recover by executing the task synchronously.
        doTask(task);
    }
}

void
NodeStoreScheduler::doTask(NodeStore::Task& task)
{
    task.performScheduledTask();

    // NOTE: It feels a bit off that there are two different methods that
    // call stopped(): onChildrenStopped() and doTask().  There's a
    // suspicion that, as long as the Stoppable tree is configured
    // correctly, this call to stopped() in doTask() can never occur.
    //
    // However, until we increase our confidence that the suspicion is
    // correct, we will leave this code in place.
    if ((--m_taskCount == 0) && isStopping() && areChildrenStopped())
        stopped();
}

void
NodeStoreScheduler::onFetch(NodeStore::FetchReport const& report)
{
    if (report.wentToDisk)
        m_jobQueue->addLoadEvents(
            report.isAsync ? jtNS_ASYNC_READ : jtNS_SYNC_READ,
            1,
            report.elapsed);
}

void
NodeStoreScheduler::onBatchWrite(NodeStore::BatchWriteReport const& report)
{
    m_jobQueue->addLoadEvents(jtNS_WRITE, report.writeCount, report.elapsed);
}

}  // namespace ripple
